@@ -18,7 +18,8 @@ from quodlibet.formats import PEOPLE
 from quodlibet.util import format_rating, connect_obj
 from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.textedit import PatternEditBox
-from quodlibet.parse import XMLFromMarkupPattern
+from quodlibet.qltk import Button, Icons
+from quodlibet.pattern import XMLFromMarkupPattern
 
 
 EMPTY = _("Songs not in an album")
@@ -70,7 +71,7 @@ class Preferences(qltk.UniqueWindow):
             return
         super(Preferences, self).__init__()
         self.set_border_width(12)
-        self.set_title(_("Album List Preferences") + " - Quod Libet")
+        self.set_title(_("Album List Preferences"))
         self.set_default_size(420, 380)
         self.set_transient_for(qltk.get_top_parent(browser))
         # Do this config-driven setup at instance-time
@@ -113,7 +114,7 @@ class Preferences(qltk.UniqueWindow):
         box.pack_start(f, True, True, 0)
 
         main_box = Gtk.VBox(spacing=12)
-        close = Gtk.Button(stock=Gtk.STOCK_CLOSE)
+        close = Button(_("_Close"), Icons.WINDOW_CLOSE)
         close.connect('clicked', lambda *x: self.destroy())
         b = Gtk.HButtonBox()
         b.set_layout(Gtk.ButtonBoxStyle.END)
