@@ -10,21 +10,20 @@ from gi.repository import Gtk, GLib
 
 from quodlibet import config
 
-from quodlibet.browsers._base import Browser
-from quodlibet.parse import Query
+from quodlibet.browsers import Browser
+from quodlibet.query import Query
 from quodlibet.qltk.songlist import SongList
 
 
-class EmptyBar(Gtk.VBox, Browser):
+class EmptyBar(Browser):
     """A browser that the user only interacts with indirectly, via the
     Filter menu. The VBox remains empty."""
 
-    __gsignals__ = Browser.__gsignals__
-
     name = _("Disable Browser")
     accelerated_name = _("_Disable Browser")
+    keys = ["Disabled", "EmptyBar"]
     priority = 0
-    in_menu = False
+    is_empty = True
 
     def pack(self, songpane):
         container = Gtk.VBox(spacing=6)
@@ -38,13 +37,13 @@ class EmptyBar(Gtk.VBox, Browser):
 
     def __init__(self, library):
         super(EmptyBar, self).__init__()
-        self._text = ""
-        self._filter = None
+        self._text = u""
+        self._query = None
         self._library = library
 
     def active_filter(self, song):
-        if self._filter is not None:
-            return self._filter(song)
+        if self._query is not None:
+            return self._query.search(song)
         else:
             return True
 
@@ -52,6 +51,9 @@ class EmptyBar(Gtk.VBox, Browser):
         self._text = text
         self.activate()
 
+    def get_filter_text(self):
+        return self._text
+
     def save(self):
         config.set("browsers", "query_text", self._text.encode('utf-8'))
 
@@ -68,21 +70,16 @@ class EmptyBar(Gtk.VBox, Browser):
 
     def _get_songs(self):
         try:
-            self._filter = Query(self._text, star=SongList.star).search
+            self._query = Query(self._text, star=SongList.star)
         except Query.error:
             pass
         else:
-            if Query.match_all(self._text):
-                songs = self._library.values()
-                self._filter = None
-            else:
-                songs = filter(self._filter, self._library)
-            return songs
+            return self._query.filter(self._library)
 
     def activate(self):
         songs = self._get_songs()
         if songs is not None:
-            GLib.idle_add(self.emit, 'songs-selected', songs, None)
+            GLib.idle_add(self.songs_selected, songs)
 
     def can_filter_text(self):
         return True
