@@ -15,8 +15,10 @@ from threading import Thread
 
 from gi.repository import Gtk, GLib
 
-from quodlibet import const, config, util, qltk
+import quodlibet
+from quodlibet import config, util, qltk
 from quodlibet.qltk.entry import UndoEntry
+from quodlibet.qltk import Icons
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 
 try:
@@ -190,12 +192,13 @@ class LastFMSyncCache(object):
             song['~#added'] = min(song['~#added'], stats['added'])
 
 
-class LastFMSyncWindow(Gtk.Dialog):
+class LastFMSyncWindow(qltk.Dialog):
     def __init__(self, parent):
         super(LastFMSyncWindow, self).__init__(
-                _("Last.fm Sync"), parent, buttons=(
-                    Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
-                    Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT))
+                _("Last.fm Sync"), parent)
+        self.add_button(_("_Cancel"), Gtk.ResponseType.REJECT)
+        self.add_icon_button(_("_Save"), Icons.DOCUMENT_SAVE,
+                             Gtk.ResponseType.ACCEPT)
         self.set_border_width(5)
         self.set_default_size(300, 100)
 
@@ -225,9 +228,9 @@ class LastFMSync(SongsMenuPlugin):
     PLUGIN_NAME = _("Last.fm Sync")
     PLUGIN_DESC = _("Updates your library's statistics from your "
                     "Last.fm profile.")
-    PLUGIN_ICON = 'gtk-refresh'
+    PLUGIN_ICON = Icons.VIEW_REFRESH
 
-    CACHE_PATH = os.path.join(const.USERDIR, "lastfmsync.db")
+    CACHE_PATH = os.path.join(quodlibet.get_user_dir(), "lastfmsync.db")
 
     def runner(self, cache):
         changed = True
