@@ -15,10 +15,9 @@ from gi.repository import Gtk, Pango, GLib
 
 from quodlibet import util
 from quodlibet import config
-from quodlibet import const
 from quodlibet.pattern import Pattern
 from quodlibet.qltk.views import TreeViewColumnButton
-from quodlibet.util.path import fsdecode, unexpand
+from quodlibet.util.path import fsdecode, unexpand, fsnative
 from quodlibet.formats._audio import FILESYSTEM_TAGS
 
 
@@ -82,8 +81,6 @@ class SongListColumn(TreeViewColumnButton):
 class TextColumn(SongListColumn):
     """Base text column"""
 
-    __label = Gtk.Label().create_pango_layout("")
-
     def __init__(self, tag):
         super(TextColumn, self).__init__(tag)
 
@@ -93,6 +90,10 @@ class TextColumn(SongListColumn):
 
         self.set_clickable(True)
 
+    @util.cached_property
+    def _layout(self):
+        return Gtk.Label().create_pango_layout("")
+
     def _cell_width(self, text, pad=8):
         """Returns the column width needed for the passed text"""
 
@@ -100,8 +101,8 @@ class TextColumn(SongListColumn):
         return self._text_width(text) + pad + cell_pad
 
     def _text_width(self, text):
-        self.__label.set_text(text, -1)
-        return self.__label.get_pixel_size()[0]
+        self._layout.set_text(text, -1)
+        return self._layout.get_pixel_size()[0]
 
     def _cdf(self, column, cell, model, iter_, user_data):
         """CellRenderer cell_data_func"""
@@ -175,7 +176,8 @@ class DateColumn(WideTextColumn):
             else:
                 format_ = "%x"
             stamp = time.localtime(stamp)
-            text = time.strftime(format_, stamp).decode(const.ENCODING)
+            encoding = util.get_locale_encoding()
+            text = time.strftime(format_, stamp).decode(encoding)
             cell.set_property('text', text)
 
 
@@ -201,10 +203,11 @@ class FSColumn(WideTextColumn):
         self._render.set_property('ellipsize', Pango.EllipsizeMode.MIDDLE)
 
     def _cdf(self, column, cell, model, iter_, user_data):
-        value = model.get_value(iter_).comma(self.header_name)
+        values = model.get_value(iter_).list(self.header_name)
+        value = values[0] if values else fsnative(u"")
         if not self._needs_update(value):
             return
-        cell.set_property('text', unexpand(fsdecode(value)))
+        cell.set_property('text', fsdecode(unexpand(value)))
 
 
 class PatternColumn(WideTextColumn):
